/*
 * fedit.h --- FEdit main include file
 * 
 * Copyright (c) 2019 Mariano Alvarez Fernandez (malfer@telefonica.net)
 *
 * This file is part of FEdit, a visual font editor for the MGRX font format
 *
 * FEdit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * FEdit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with FEdit. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#define MAXLEN_FNAME 120
#define MAXBOX_H 400
#define MAXBOX_W 400

typedef struct {
    char *dot;
    int width, height;
    int dotsize;
    int dotfill;
    int dotselec;
    int hline, vline;
    int xorg, yorg;
    int edited;
    GrColor fillcolor;
    GrColor dotoncolor;
    GrColor dotoffcolor;
    GrColor dotselcolor;
    GrColor hlinecolor;
    GrColor vlinecolor;
} DotMatrix;

DotMatrix *DotMatrixCreate(int width, int height, int dotsize, int dotfill);
void DotMatrixSetPos(DotMatrix *dm, int xorg, int yorg);
void DotMatrixSetHLine(DotMatrix *dm, int hline);
void DotMatrixSetVLine(DotMatrix *dm, int vline);
void DotMatrixIncrVLine(DotMatrix *dm, int dir);
void DotMatrixSetColors(DotMatrix *dm, GrColor fill, GrColor doton,
                        GrColor dotoff, GrColor dotsel);
void DotMatrixSetLineColors(DotMatrix *dm, GrColor hline, GrColor vline);
void DotMatrixSetAllDots(DotMatrix *dm, int onoff);
void DotMatrixSetDot(DotMatrix *dm, int x, int y, int onoff);
int DotMatrixSwtichDot(DotMatrix *dm, int x, int y);
int DotMatrixGetDot(DotMatrix *dm, int x, int y);
int DotMatrixCalcDotByGCoord(DotMatrix *dm, int gx, int gy, int *x, int *y, int m);
void DotMatrixMoveX(DotMatrix *dm, int dir);
void DotMatrixMoveY(DotMatrix *dm, int dir);
void DotMatrixPaint(DotMatrix *dm);
void DotMatrixRePaintDots(DotMatrix *dm);
void DotMatrixRePaintLines(DotMatrix *dm, int show);
void DotMatrixPaintDot(DotMatrix *dm, int x, int y);
void DotMatrixSetSelect(DotMatrix *dm, int x, int y, int paint);
void DotMatrixMoveRelSelect(DotMatrix *dm, int row, int col, int paint);
int DotMatrixSwitchSelect(DotMatrix *dm, int paint);
void DotMatrixReflectV(DotMatrix *dm);
void DotMatrixReflectH(DotMatrix *dm);
void DotMatrixFill(DotMatrix *dm, int x, int y);
void DotMatrixDestroy(DotMatrix *dm);

typedef struct {
    int inited;
    int width;
    int bmprowsize;
    char *bmp;
} FontWorkGlyph;

typedef struct {
    GrFontHeader h;
    int maxwidth;
    int maxglyphs;
    FontWorkGlyph *gly;
    char *bmp;
    GrContext *wctx;
    int glyphselec;
    int xcent, ycent, yorg;
    int glyphxrow;
    int numlines;
    int screenlines;
    int firstline;
    GUITile *t;
    GrColor bgcolor;
    GrColor fgcolor;
    GrColor selectcolor;
    GrColor noinitcolor;
} FontWork;

FontWork * FontWorkCreate(GrFont *font, int maxglyphs);
int FontWorkAppend(FontWork *dfwk, GrFont *sf, int fsglyph, int lsglyph,
                   int fdglyph, int xpivot, int ypivot);
FontWork * FontWorkResizeBox(FontWork *fwk, int top, int left,
                             int right, int bottom);
FontWork * FontWorkConvToFixed(FontWork *fwk);
void FontWorkLoadGlyph(FontWork *fwk, GrFont *font, int glyph);
void FontWorkSetDims(FontWork *fwk, GUITile *t);
void FontWorkSetColors(FontWork *fwk, GrColor bg, GrColor fg,
                       GrColor select, GrColor noinit);
void FontWorkPaintGlyphXY(FontWork *fwk, int glyph, int x, int y, int selec, int blit);
void FontWorkPaintGlyph(FontWork *fwk, int glyph, int selec, int blit);
void FontWorkPaint(FontWork *fwk);
void FontWorkSetSelect(FontWork *fwk, int glyph, int paint);
void FontWorkMoveSelect(FontWork *fwk, int row, int col, int paint);
void FontWorkMovePagSelect(FontWork *fwk, int direc, int paint);
void FontWorkScrollSelect(FontWork *fwk, int newfirst, int paint);
int FontWorkCalcGlyphByGCoord(FontWork *fwk, int gx, int gy, int *glyph);
void FontWorkGetRealSelectedXYPos(FontWork *fwk, int *x, int *y);
void FontWorkCopyGlyph(FontWork *fwk, int sglyph, int dglyph);
GrFont * FontWorkGenFont(FontWork *fwk);
int FontWorkCheckUninitedInRange(FontWork *fwk, int fglyph, int lglyph);
int FontWorkCheckUninitedInFontRange(FontWork *fwk);
int FontWorkCheckInitedOutOfRange(FontWork *fwk, int fglyph, int lglyph);
int FontWorkCheckValidRange(FontWork *fwk, int fglyph, int lglyph);
int FontWorkCheckGlyphOk(FontWork *fwk, int glyph);
void FontWorkRecalMeanWidth(FontWork *fwk);
void FontWorkDestroy(FontWork *fwk);

void ShowError(char *s);
void ShowError2(char *s1, char *s2);
void ShowError2cat(char *s1, char *s2);
void ShowErrorReqBoxInvalid(int width, int height);
int WarnContinue(char *s1, char *s2);
int ConfirmContinue(char *s1, char *s2);

int FPropDlgCreate(void);
void FPropDlgDestroy(void);
void FPropDlgSetData(FontWork *fwk);
void FPropDlgGetData(FontWork *fwk);
int FPropDlgRun(void);

int FSaveDlgCreate(void);
void FSaveDlgDestroy(void);
void FSaveDlgSetData(char *output, int fglyph, int lglyph);
void FSaveDlgGetData(char *output, int *fglyph, int *lglyph);
int FSaveDlgRun(void);

int FLoadDlgCreate(void);
void FLoadDlgDestroy(void);
void FLoadDlgSetData(char *fname);
void FLoadDlgGetData(char *fname);
int FLoadDlgRun(void);

int FLoheDlgCreate(void);
void FLoheDlgDestroy(void);
void FLoheDlgSetData(char *fname, int maxglyph,
                     int fdglyph, double zoom, int thick);
void FLoheDlgGetData(char *fname, int *maxglyph,
                     int *fdglyph, double *zoom, int *thick);
int FLoheDlgRun(void);

int FApenDlgCreate(void);
void FApenDlgDestroy(void);
void FApenDlgSetData(char *output, int fsglyph, int lsglyph,
                     int fdglyph, int xpivot, int ypivot);
void FApenDlgGetData(char *output, int *fsglyph, int *lsglyph,
                     int *fdglyph, int *xpivot, int *ypivot);
int FApenDlgRun(void);

int URangDlgCreate(void);
void URangDlgDestroy(void);
void URangDlgSetData(int fglyph, int lglyph);
void URangDlgGetData(int *fglyph, int *lglyph);
int URangDlgRun(void);

int CRangDlgCreate(void);
void CRangDlgDestroy(void);
void CRangDlgSetData(int fsglyph, int lsglyph, int fdglyph);
void CRangDlgGetData(int *fsglyph, int *lsglyph, int *fdglyph);
int CRangDlgRun(void);

int RRangDlgCreate(void);
void RRangDlgDestroy(void);
void RRangDlgSetData(int fglyph, int lglyph);
void RRangDlgGetData(int *fglyph, int *lglyph);
int RRangDlgRun(void);

int RsBoxDlgCreate(void);
void RsBoxDlgDestroy(void);
void RsBoxDlgSetData(int top, int left, int right, int bottom);
void RsBoxDlgGetData(int *top, int *left, int *right, int *bottom);
int RsBoxDlgRun(void);

int RSizeDlgCreate(void);
void RSizeDlgDestroy(void);
void RSizeDlgSetData(int nwidth, int nheight, int fglyph, int lglyph);
void RSizeDlgGetData(int *nwidth, int *nheight, int *fglyph, int *lglyph);
int RSizeDlgRun(void);
